<?php

if (!defined('ABSPATH')) {
    exit;
}

class WC_rupantorpay_Gateway extends WC_Payment_Gateway
{
    public function __construct()
    {
        $this->id = 'rupantorpay';
        $this->icon = plugins_url('/assets/logo.png', __FILE__);
        $this->has_fields = false;
        $this->method_title = __('rupantorpay', 'rupantorpay');
        $this->method_description = __('Accept Bangladeshi payments via multiple gateways including bKash, Nagad, Rocket, and more.<br> <a href="https://rupantorpay.com" target="_blank">Sign up for a rupantorpay account</a>', 'rupantorpay');
        $this->supports = array('products');
        
        $this->init_form_fields();
        $this->init_settings();
        
        $this->title = $this->get_option('title');
        $this->description = $this->get_option('description');
        $this->enabled = $this->get_option('enabled');
        
        add_action('woocommerce_update_options_payment_gateways_' . $this->id, array($this, 'process_admin_options'));
        add_action('woocommerce_api_' . strtolower(get_class($this)), array($this, 'handle_webhook'));
    }
    public function init_form_fields()
    {
        $this->form_fields = require plugin_dir_path(__FILE__) . 'class-wc-rupantorpay-settings.php';
    }
       public function process_payment($order_id)
{
    global $woocommerce;
    $order = wc_get_order($order_id);
    $current_user = wp_get_current_user();
    $total = $order->get_total();

    if ($order->get_currency() == 'USD') {
        $total = $total * $this->get_option('currency_rate');
    }
    if ($order->get_status() != 'completed') {
        $order->update_status('pending', esc_html__('Customer is being redirected to RupantorPay', 'rupantorpay'));
    }
    $data = array(
         "fullname"    => !empty($current_user->user_firstname) ? $current_user->user_firstname : $current_user->display_name,
        "email"       => $current_user->user_email,
        "amount"      => (float) $total,
        "webhook_url" => site_url('/?wc-api=wc_rupantorpay_gateway&order_id=' . $order->get_id()),
        "success_url" => $this->get_return_url($order),
        "cancel_url"  => wc_get_checkout_url()
    );
    $headers = array(
        'Content-Type' => 'application/json',
        'X-API-KEY'    => $this->get_option('apikeys'),
    );
    $url = $this->get_option('payment_site') . "api/payment/checkout";
    $response = $this->create_payment($url, $data, $headers);
    $data = json_decode($response, true);
    if (isset($data['payment_url'])) {
        return array(
            'result'   => 'success',
            'redirect' => $data['payment_url']
        );
    } else {
        wc_add_notice(__('Payment error: ' . $data['message'], 'rupantorpay'), 'error');
        return;
    }
}
public function create_payment($url, $data, $headers)
{
    $args = array(
        'headers' => $headers,
        'body'    => json_encode($data),
        'timeout' => 10,
    );

    $response = wp_remote_post($url, $args);

    if (is_wp_error($response)) {
        return json_encode(["status" => 0, "message" => "Request failed"]);
    }

    return wp_remote_retrieve_body($response);
}
public function update_order_status($order)
{
    if (!isset($_REQUEST['transactionId'])) {
        wp_die(esc_html__('Invalid request. Transaction ID missing.', 'rupantorpay'));
    }

    $transactionId = sanitize_text_field(wp_unslash($_REQUEST['transactionId']));
    if (isset($_REQUEST['status'])) {
        $status = sanitize_text_field(wp_unslash($_REQUEST['status']));
        if ($status === 'completed' || $status === '2') {
            $data = array(
                "transaction_id" => $transactionId,
            );
            $headers = array(
                'Content-Type' => 'application/json',
                'X-API-KEY'    => $this->get_option('apikeys'),
            );

            $url = $this->get_option('payment_site') . "api/payment/verify-payment";
            $response = $this->create_payment($url, $data, $headers);
            $data = json_decode($response, true);
            if ($order->get_status() != 'completed') {
                if (isset($data['status']) && ($data['status'] == "COMPLETED" || $data['status'] == "2")) {
                     $transaction_id = $data['trx_id'] ?? $data['transaction_id'];
                    $amount = $data['amount'];
                    $sender_number = $data['email'];
                    $payment_method = 'rupantorpay';
                    $has_digital_products = false;
                    foreach ($order->get_items() as $item) {
                        $product = $item->get_product();
                        if ($product && $product->is_downloadable()) {
                            $has_digital_products = true;
                            break;
                        }
                    }
                    if ($has_digital_products) {
                        $order->update_status(
                            $this->get_option('digital_product_status'),
                            sprintf(
                                __('rupantorpay payment was successfully completed. Payment Method: %1$s, Amount: %2$s, Transaction ID: %3$s, Email: %4$s', 'rupantorpay'),
                                $payment_method,
                                $amount,
                                $transaction_id,
                                $sender_number
                            )
                        );
                    } else {
                        $order->update_status(
                            $this->get_option('physical_product_status'),
                            sprintf(
                                __('rupantorpay payment was successfully processed. Payment Method: %1$s, Amount: %2$s, Transaction ID: %3$s, Email: %4$s', 'rupantorpay'),
                                $payment_method,
                                $amount,
                                $transaction_id,
                                $sender_number
                            )
                        );
                    }

                    $order->reduce_order_stock();
                    $order->payment_complete($transaction_id);
                    return true;
                } else {
                    $order->update_status('on-hold', esc_html__('rupantorpay payment is on hold. Transaction ID not found. Please check it manually.', 'rupantorpay'));
                    return true;
                }
            }
        } elseif ($status === 'pending' || $status === '0') {
            $order->update_status('pending', esc_html__('rupantorpay payment is pending.', 'rupantorpay'));
            return true;
        } elseif ($status === 'cancelled' || $status === '3') {
            $order->update_status('cancelled', esc_html__('rupantorpay payment was cancelled.', 'rupantorpay'));
            return true;
        } else {
            $order->update_status('on-hold', esc_html__('rupantorpay payment status is unknown. Please check it manually.', 'rupantorpay'));
            return true;
        }
    } else {
    }
}
public function handle_webhook()
{
    if (empty($_GET['order_id']) || !is_numeric($_GET['order_id'])) {
        wp_die(esc_html__('Invalid request.', 'rupantorpay'));
    }
    $order_id = sanitize_text_field(wp_unslash($_GET['order_id']));
    $order = wc_get_order($order_id);
    if ($order) {
        $this->update_order_status($order);
    }
    status_header(200);
    echo wp_json_encode(['message' => esc_html__('Webhook received and processed.', 'rupantorpay')]);
    exit();
}
}
